module.exports = function (RED) {
  'use strict'
  const TVIoT = require('@teamviewer/teamviewer-iot-mqtt-sdk');
  const fs = require('fs');
  const path = require('path');
  require('events').EventEmitter.defaultMaxListeners = 1000;
  let clientConnector;
  let sensorList = [];

  function ClientNode (config) {
	RED.nodes.createNode(this, config);

	this.config = {
	  pathCS: config.pathCS,
	  host: config.host,
	  port: config.port,
	  provision: config.provision,
	  pathCA: config.pathCA,
	  pathLog: config.pathLog,
	  logLevel: config.logLevel,
	};
	this.connId = null;
	this.sensor_list = [];
	const node = this;

	const TV_IOT_CONFIG = {
	  'MqttHost': this.config.host,
	  'MqttPort': this.config.port, // 18884
	  'MqttProvisionPort': this.config.provision, // 18883
	  'CaFile': this.config.pathCA,
	  'CertFolder': this.config.pathCS,
	  'logFilePath' : this.config.pathLog,
	  'logLevel' : this.config.logLevel
	};

	const clientId = node.context().get('clientid')||null;;
	const certFile = `${TV_IOT_CONFIG.CertFolder}/cert-${clientId}.pem`;
	const keyFile = `${TV_IOT_CONFIG.CertFolder}/key-${clientId}.pem`;
	if(fs.existsSync(certFile) && fs.existsSync(keyFile)) {
	  // ClientId exists
	  node.trace('OLDCONNECTION ID ' + clientId);
	  clientConnector = new TVIoT(TV_IOT_CONFIG);
	  clientConnector.connect_api(certFile, keyFile);
	  clientConnector.on('connected', onConnected);
	} else {
	  // ClientId not exists
	  clientConnector = new TVIoT(TV_IOT_CONFIG);
	  clientConnector.provision();
	  clientConnector.on('clientCreated', (connectorId) => {
	node.connId = connectorId;
		clientConnector.disconnect_api();
		node.trace('CONNECTION ID ' + node.connId);
		clientConnector = new TVIoT(TV_IOT_CONFIG);
		clientConnector.connect_api(clientConnector.clientCertFile(node.connId), clientConnector.clientKeyFile(node.connId));
		clientConnector.on('connected', onConnected);
	  });
	}

	function onConnected(connectorId) {
	  node.connId = connectorId;
	  node.debug('Connected securely to the API: ' + node.connId );
	  node.context().set('clientid', node.connId)
	  clientConnector.list_sensors(node.connId, (sensors) => {
		  sensorList = [];
		  node.trace('SENSOR LIST----- ' + sensors);
		  node.sensor_list = JSON.parse(sensors);
		  node.emit('clientConnected', node.sensor_list);
		  saveSensors(0);
		},
		errorCallback);
	}

	this.pushSensor = function(flag, sensorid, sensorname, sensoroldname){
	  let sensorData = {'flag': flag, 'id': sensorid, 'name': sensorname, 'oldname': sensoroldname};
	  sensorList.push(sensorData);
	}

	function saveSensors(counter) {
	  if (sensorList.length > counter){
		let sensor = sensorList[counter];
		if (sensor.flag == 'new') {
		  // CREATE SENSOR - create_sensor(sensorName, callback, errorCallback)
		  clientConnector.create_sensor(sensor.name, (sensor_response) => {
			  node.trace('CREATE SENSOR -------- ' + sensor_response);
			  let existSensor = JSON.parse(sensor_response);
			  node.debug('new Sensor: ' + sensor.name + '(' + existSensor.sensorId + ')');
			  sensorList[counter].id = existSensor.sensorId;
			  sensorList[counter].metrics = [];
			  saveSensors(counter + 1);
			},
			errorCallback);
		} else if (sensor.flag == 'update'){
		  // UPDATE SENSOR - update_sensor(sensorId, sensorName, callback)
		  clientConnector.update_sensor(sensor.id, sensor.name, (response) => {
			node.trace('UPDATE SENSOR RESPONSE ---------', response);
			node.debug('Change the sensorname: ' + sensor.oldname + ' -> ' + sensor.name);
			saveSensors(counter + 1);
		  });
		} else {
		  saveSensors(counter + 1);
		};
	  } else {
		clientConnector.list_sensors(node.connId, (sensors) => {
			node.sensor_list = JSON.parse(sensors);
			node.emit('sensorsCreated', sensorList);
			sensorList = []
		  },
		  errorCallback);
	  }

	}

	function errorCallback(error) {
	  node.clientErrorCallback(error);
	}

	this.clientErrorCallback = function(error) {
	  node.error('Error: ' + error);
	}

	this.getClientConnector = function() {
	  return clientConnector;
	}

	this.disconnect = function() {
	  clientConnector.disconnect_api( () =>{
		node.trace(`Disconnect finished!`);
		this.connected = false;
		node.emit('clientDisconnected');
	  });
	}

	clientConnector.on('errorCreatingClient', (err) => {
	  if (err != null) {
		node.error('Error: ' + err);
		clientConnector.report_connector_error(err, () => {
		  this.disconnect();
		})
	  }
	})

	clientConnector.on('error', (err) => {
	  if (err != null) {
		node.error('Error: ' + err);
	  }
	})

	clientConnector.on('offline', (err) => {
	  if (err != null) {
		node.error('Offline: ' + err);
	  }
	})

	clientConnector.on('close', (err) => {
	  if (err != null) {
		node.error('Close: ' + err);
	  }
	})

	clientConnector.on('problemConnectingClient', (err) => {
	  if (err != null) {
		node.error('problemConnectingClient: ' + err);
	  }
	})

  }

  RED.nodes.registerType ('tv_iot_client', ClientNode);

}
