module.exports = function (RED) {
  'use strict'

  function TVIoT_OutNode (config) {
    RED.nodes.createNode(this, config);

    this.status({fill:"red", shape:"ring", text:"disconnect"});
    this.sensor = RED.nodes.getNode(config.tv_iot_sensor);
    this.sensorId = null;
    this.metricId = null;
    this.payload = null;
    this.metricold = null;

    const node = this;

    if (this.sensor) {
      this.sensor.on('sensorCreated', createMetric);
      this.sensor.on('metricsCreated', (metricListNew) => {
        metricListNew.forEach(function (metric) {
          if (node.context().get('metricname') == metric.name){
            node.context().set('metricid', metric.matchingId);
            node.context().set('annotation', metric.valueAnnotation||'');
          }
        });
        metricListNew = [];
      });
      this.sensor.on('metricNew', newMetric);
      this.sensor.on('metricUpdate', checkUpdate);

      this.sensor.once('clientConnected', () => {
        this.status({fill:"green", shape:"dot", text:"connected"});
      });

      this.sensor.once('clientDisconnected', () => {
        this.status({fill:"red", shape:"ring", text:"disconnect"});
      });

      function createMetric(sensor_list, callback) {
        function returnback(){
          callback();
        }
        if (node.sensor) {
          node.sensorId = node.sensor.sensorid;
          // iterate over each element in the sensor list
          node.sensor.create_metrics(sensor_list, node.context().get('metricid')||null, config.metricname, returnback);
        }
      }

      function newMetric(metricname, callback){
        function returnback(){
          callback();
        }
        if (config.metricname == metricname){
          node.sensor.countMetrics();
          node.context().set('metricname', config.metricname);
          if (config.predefined) {
            node.sensor.pushMetricData('new', true, undefined, config.metricname, config.valueunit, undefined, undefined, returnback);
          }else{
            node.context().set('annotation', config.annotation);
            node.sensor.pushMetricData('new', false, undefined, config.metricname, undefined, config.annotation, config.valuetype, returnback);
          }
        }
      }

      function checkUpdate(metricId, callback){
        function returnback(){
          callback();
        }
        node.metricId = node.context().get('metricid')||null;
        if (node.metricId == metricId){
          node.sensor.countMetrics();
          node.metricold = node.context().get('metricname')||null;
          node.annotation = node.context().get('annotation')||'';
          let correct;
          correct = true;
          if (!(!config.predefined && node.annotation != '') && !(config.predefined && node.annotation == '')) {
            correct = false;
            node.error('Error: Change the value type from predefined to custom and the other way around is not possible. (metric: ' + config.metricname + ' - sensor:' + node.sensorId + ')')
            returnback();
          }
          if (correct && (config.metricname != node.metricold || config.annotation != node.annotation)) {
            node.context().set('metricname', config.metricname);
            if (config.predefined) {
              node.context().set('annotation', '');
              node.sensor.pushMetricData('update', true, node.metricId, config.metricname, config.valueunit, undefined, undefined, returnback);
            }else{
              node.context().set('annotation', config.annotation);
              node.sensor.pushMetricData('update', false, node.metricId, config.metricname, undefined, config.annotation, config.valuetype, returnback);
            }
          }
        }
      }

      this.on('input', function (msg, done) {
        node.payload = msg.payload;
        if (node.payload != null) {
          node.sensor.pushMetricValues(node.context().get('metricid'), node.payload, done);
        }
      })


      this.on('close', function(removed, done) {
        if (removed) {
          const metricList = {
                "metrics": [
                    {
                        "metricId": node.context().get('metricid')
                    }
                ]
              };

          function callback(){
            node.debug('Deleted metric: ' + config.metricname);
            done();
          }

          node.sensor.deleteMetric(JSON.stringify(metricList), callback);
        }else{
          done();
        }
      });

    }

  }

  RED.nodes.registerType('tv_iot_out', TVIoT_OutNode);
}
