module.exports = function (RED) {
  'use strict'

  function SensorNode (config) {
    RED.nodes.createNode(this, config);
    this.sensorname = config.sensorname;
    this.sensorold = null;
    let metricListNew = [];
    let metricListUpdate = [];
    let metricListValues = [];
    let clientConnector;
    this.metric_list = [];

    this.client = RED.nodes.getNode(config.tv_iot_client);
    this.existSensor = null;
    this.matchingId = 0;
    this.metrics = 0;
    this.metriccount = 0;
    this.sensorid;

    const node = this;

    if (this.client) {
      this.client.on('clientConnected', clientConnected);
      this.client.on('clientDisconnected', () => {node.emit('clientDisconnected')});
      this.client.on('sensorsCreated', (sensorList) => {
        // Save the sensor-Data
        let ifexist = true;
        sensorList.forEach(function (sensor) {
          if (sensor.flag == 'new') {
            if (node.context().get('sensorname') == sensor.name){
              ifexist = false;
              node.context().set('sensorid', sensor.id);
              node.sensorid = sensor.id;
            }
          }
        });

        node.metriccount = 0;

        function returnback(){
          if (node.metriccount >= node.listenerCount('sensorCreated')){
            // Proov the metric-count -> all metrics finish -> next step
            ifexist = true;
            saveMetrics();
          }
        }
        node.emit('sensorCreated', this.client.sensor_list, returnback);
      });
    }else{
      return;
    }


    function clientConnected(sensor_list){
      node.emit('clientConnected');
      // send sensor name to the client node
      node.sensorold = node.context().get('sensorname')||null;
      node.sensorid = node.context().get('sensorid')||null;
      clientConnector = node.client.getClientConnector();
      getSensor(sensor_list, 0);
    }


        /** get sensor by name from sensors list or
     *  create sensor if it does not exists to push metrics data for it */
    function getSensor(sensor_list, counter) {
      if (sensor_list.length == counter) {
        // no sensor exist
        node.context().set('sensorname', node.sensorname);
        node.metrics = 0;
        node.client.pushSensor('new', undefined, node.sensorname, undefined);
      } else {
        node.existSensor = sensor_list[counter];
        if (node.existSensor.name == node.sensorold) {
          node.metrics = sensor_list[counter].metrics.length;
          node.context().set('sensorname', node.sensorname);
          node.context().set('sensorid', node.existSensor.sensorId);
          node.sensorid = node.existSensor.sensorId;
          // changing the name if possible
          if (node.existSensor.name != node.sensorname) {
            // update sensor
            node.client.pushSensor('update', node.existSensor.sensorId, node.sensorname, node.sensorold);
          } else {
            // known sensor
            node.client.pushSensor('old',node.existSensor.sensorId, node.sensorname, undefined);
          }
        } else {
          getSensor(sensor_list, counter + 1);
        }
      }
    }

    this.pushMetricData = function(flag, pre, metricid, name, valueUnit, valueAnnotation, valueType, callback){
      let metricData;
      if (flag == 'new') {
        let matchingId = nextMatchingId().toString();
        if (pre == true){
          metricData = {'matchingId': matchingId, 'valueUnit': valueUnit, 'name': name};
        } else {
          metricData = {'matchingId': matchingId, 'name': name, 'valueAnnotation': valueAnnotation, 'valueType': valueType};
        }
        metricListNew.push(metricData);
      } else {
        if (pre == true){
          metricData = {'metricId': metricid, 'name': name};
        } else {
          metricData = {'metricId': metricid, 'name': name, 'valueAnnotation': valueAnnotation};
        }
        metricListUpdate.push(metricData);
      }
      callback = true;
    }

    this.pushMetricValues = function(metricid, value, done){
      clientConnector.put_metric(node.sensorid
        , JSON.stringify({metrics: [{'metricId': metricid, 'value': value}]})
        , (push_response) => {
          node.trace('PUT METRIC (' + metricid + ') RESPONSE --------- ' + push_response);
          done();
      },
      errorCallback);
    }

    this.create_metrics = function(sensor_list, metricId, metricname, callback){
      let found = false;
      let i;
      function returnback(){
        i = sensor_list.length;
      }
      //node.sensorid = node.context().get('sensorid');
      for (i = 0; i < sensor_list.length; i++){
        // look for the entry with a matching sensorId
        if (sensor_list[i].sensorId == node.sensorid){
          found = true;
          if (sensor_list[i].metrics != ''){
            if (metricId == null) {
              node.emit('metricNew', metricname, returnback);
            } else {
              // check Update
              node.emit('metricUpdate', metricId, returnback);
            }
          }else{
            node.emit('metricNew', metricname, returnback);
          }
        }
      }
      if (!found && i == sensor_list.length){
        node.emit('metricNew', metricname, returnback);
      }else{
        if (i >= sensor_list.length){
          callback();
        }
      }
    }

    function saveMetrics() {
        if (metricListNew.length > 0){
            clientConnector.create_metrics(node.sensorid
              , JSON.stringify({metrics: metricListNew})
              , (metric_response) => {
                  node.trace(`CREATE METRICS ------- ${metric_response}`);
                  let metrics = JSON.parse(metric_response);
                  // MatchingId -> MetricId
                  metrics.forEach(function (metric) {
                    let i;
                    for (i=0; i<metricListNew.length; i++){
                      if (metricListNew[i].matchingId == metric.matchingId){
                        metricListNew[i].matchingId = metric.metricId;
                      }
                    };
                  });
                  node.emit('metricsCreated', metricListNew);
                  metricListNew = [];
              }, errorCallback);
        }

        if (metricListUpdate.length > 0){
          clientConnector.update_metrics(node.sensorid
            , JSON.stringify({metrics: metricListUpdate})
            , (response) => {
              node.trace('UPDATE METRICS RESPONSE --------- ' + response);
              metricListUpdate = [];
            }, errorCallback);
        }
    }

    this.deleteMetric = function(metricList, callback){
      clientConnector.delete_metrics(node.sensorid, metricList, errorCallback);
      callback();
    }

    this.getClientConnector = function() {
      return clientConnector;
    }

    function nextMatchingId() {
      node.matchingId = node.matchingId + 1;
      return node.matchingId;
    }

    this.countMetrics = function() {
      node.metriccount = node.metriccount + 1;
    }

    this.disconnect = function() {
      node.client.disconnect();
    }

    function errorCallback(error) {
      node.sensorErrorCallback(error);
    }

    this.sensorErrorCallback = function(error) {
      node.client.clientErrorCallback(error);
    }

    function deleteSensor(done) {
      /** DELETE SENSOR - delete_sensor(sensorId, callback, errorCallback)*/
      clientConnector.delete_sensor(node.context().get('sensorid'), (response) => {
        node.trace('SENSOR DELETED -----', response);
        node.debug('Deleted sensor: ' + node.sensorname);
        done();
      }, errorCallback);
    }

    this.on('close', function(removed, done) {
      if (removed) {
        node.trace('Deleting sensor: ' + node.sensorname);
        // This node has been deleted
        // Delete all metrics
        deleteSensor(done);

      }else{
        done();
      }
    });

  }
  RED.nodes.registerType ('tv_iot_sensor', SensorNode);
}
